/* log.c */

/*
  SquidClamav - A Clamav Antivirus Redirector for Squid
  
  This project is a modified version of the excellent Squirm Redirector
  for Squid Maintained by Chris Foote, and copyrighted
  as follow :

  Copyright (C) 1998 Chris Foote & Wayne Piekarski

  The original Squirm version used was squirm-1.0betaB. Some other parts
  are cut and paste from the ex1.c program given in the ClamAv distribution
  and are copyrighted: Copyright (C) 2002 - 2004 Tomasz Kojm

  All other code: Copyright (C) 2005-2008 Gilles Darold


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Please see the file GPL in this directory for full copyright
  information.
*/


#include"squidclamav.h"
#include"log.h"
#include"util.h"
#include<stdio.h>
#include<time.h>
#include<stdlib.h>
#include<string.h>
#include<sys/types.h>
#include<unistd.h>

static char *get_date(void);

extern int interactive;       /* from main.c */

void logit(char *filename, char *format, ...)
{
  FILE *log;
  char *date_str = NULL;
  char *msg = NULL;
  va_list args;

  
  va_start(args, format);
  msg = (char *)malloc(sizeof(char)*NORMAL_BUFF);
  if (vsnprintf(msg, NORMAL_BUFF, format, args) < 0) {
    /* Can't format log, so just return */
    return;
  }
  va_end(args);

  date_str = get_date();
  if ( (interactive != 0) || (filename == NULL) ) {
    fprintf(stderr, "%s %s", date_str, msg);
    if (fflush(stderr) != 0)
       fprintf(stderr, "%s ERROR %s", date_str, "Error flushing log stream\n");
  } else {
    /* ensure that this is the good file (i-e not symlink nor device */
    if (isPathSecure(filename) == 0) {
	    log = fopen(filename, "at");
	    if (log == NULL) {
		fprintf(stderr, "%s ERROR %s", date_str, "Error opening log stream\n");
		fprintf(stderr, "%s %s", date_str, msg);
	    } else {
		    
		    fprintf(log, "%s [%d] %s", date_str, (int)getpid(), msg);
		    if (fflush(log) != 0)
			fprintf(stderr, "%s ERROR %s", date_str, "Error flushing log stream\n");
		    if (fclose(log) != 0)
			fprintf(stderr, "%s ERROR %s", date_str, "Error closing log stream\n");
		}
	} else {
	    fprintf(stderr, "%s ERROR %s", date_str, "Error log file is unsecure\n");
	}
  }
  xfree(date_str);
  xfree(msg);
}


char *get_date(void)
{
  time_t tp;
  char *ascitime = NULL;
  char *s = NULL;
  tp = time(NULL);

  ascitime = ctime(&tp);
 
  if (ascitime != NULL) { 
	  s = (char *) malloc (sizeof (char) * MAX_CHAR);
	  xstrncpy(s, ascitime, MAX_CHAR);
	  chomp(s);
  }
 
  return s;
}

