/* scconfig.c */

/*
  SquidClamav - A Clamav Antivirus Redirector for Squid
  
  This project is a modified version of the excellent Squirm Redirector
  for Squid Maintained by Chris Foote, and copyrighted
  as follow :

    Copyright (C) 1998 Chris Foote & Wayne Piekarski

  The original Squirm version used was squirm-1.0betaB. Some other part
  are cut and paste from the ex1.c program given in the ClamAv distribution
  and are copyrighted: Copyright (C) 2002 - 2004 Tomasz Kojm

  All other code: Copyright (C) 2005-2008 Gilles Darold

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Please see the file GPL in this directory for full copyright
  information.
*/

#include<string.h>
#include<stdio.h>
#include<stdlib.h>
#include<ctype.h>
#include"squidclamav.h"
#include"log.h"
#include"util.h"


extern int bridge_mode;		/* from main.c */
extern char *alternate_config;	/* from main.c */
extern int check_content_type;	/* from main.c */
extern int has_regex;		/* from main.c */
extern char *redirect_url;	/* from main.c */
extern char *squidguard;	/* from main.c */
extern int debug;		/* from main.c */
extern int force;		/* from main.c */
extern int statit;		/* from main.c */
extern char *log_file;		/* from main.c */
extern char *proxy;		/* from main.c */
extern char *clamd_ip;		/* from main.c */
extern char *clamd_port;	/* from main.c */
extern char *clamd_local;	/* from main.c */
extern int timeout;		/* from main.c */
extern char *useragent;		/* from main.c */
extern int maxredir;		/* from main.c */

/* load the squidclamav.conf into
   linked list */

void load_patterns(int reread)
{
  char *buf = NULL;
  FILE *fp  = NULL;

  /* from main.c - main(), use alternate config if given */
  if ( (alternate_config != NULL) && (isPathExists(alternate_config) == 0) ) { 
	    fp = fopen(alternate_config, "rt");
  } else if (isPathExists(CONFIG_FILE) == 0) {
    fp = fopen(CONFIG_FILE, "rt");
  }
  

  if (fp == NULL) {
    logit(log_file, "PANIC unable to open configuration file\n");
    bridge_mode = 1;
    return;
  }

 if (reread == 0) {
  if (alternate_config != NULL)
    logit(log_file, "LOG Reading configuration from %s\n", alternate_config);
  else
    logit(log_file, "LOG Reading configuration from %s\n", CONFIG_FILE);
 }

  pattc = 0; 
  buf = (char *)malloc(sizeof(char)*LOW_BUFF*2);
  while ((bridge_mode == 0) && (fgets(buf, LOW_BUFF*2, fp) != NULL)) {
      /* chop newline */
      chomp(buf);
      /* add to regex patterns array */
      if (pattc < PATTERN_ARR_SIZE) {
	      add_pattern(buf);
      } else {
	    logit(log_file, "WARNING number of regexp pattern limit reached.\n");
	    continue;
      }
  }
  xfree(buf);
  if (redirect_url == NULL) {
    logit(log_file, "PANIC No redirection URL set, going to BRIDGE mode\n");
    bridge_mode = 1;
  }
   if ((squidguard != NULL) && (reread == 0)) {
    logit(log_file, "LOG Chaining with %s\n", squidguard);
  }
   if (useragent != NULL) {
    logit(log_file, "LOG Anonymizing User-Agent: %s\n", useragent);
  }
  
  if (fclose(fp) != 0)
	logit(log_file, "ERROR Can't close configuration file\n");

  /* Set default values */
  if (proxy == NULL) {
	proxy = (char *) malloc (sizeof (char) * MAX_CHAR);
	if(proxy == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	xstrncpy(proxy, MY_PROXY, MAX_CHAR);
  }

  if (clamd_ip == NULL) {
	clamd_ip = (char *) malloc (sizeof (char) * LOW_CHAR);
	if(clamd_ip == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	xstrncpy(clamd_ip, CLAMD_SERVER, LOW_CHAR);
  }

  if (clamd_port == NULL) {
	clamd_port = (char *) malloc (sizeof (char) * LOW_CHAR);
	if(clamd_port == NULL) {
		logit(log_file, "PANIC unable to allocate memory in load_patterns()\n");
		bridge_mode = 1;
		return;
	}
	xstrncpy(clamd_port, CLAMD_PORT, LOW_CHAR);
  }

}

/*
 * Start with an array for 8 patterns, and double it 
 * each time we outgrow it. If pattern is empty (0 length),
 * or if f_matchall is already set, set f_matchall and return.
 * No use adding a pattern if all input is going to match
 * anyhow.
 */
void
add_pattern(char *s)
{
	char *first = NULL;
	char *type  = NULL;
	int stored = 0;
	int regex_flags = REG_NOSUB;

	/* skip empty and commented lines */
	if ( (xstrnlen(s, LOW_BUFF*2) == 0) || (strncmp(s, "#", 1) == 0)) return;

	/* Config file directives are construct as follow: name value */  
	type = (char *)malloc(sizeof(char)*MAX_CHAR);
	first = (char *)malloc(sizeof(char)*LOW_BUFF);
	stored = sscanf(s, "%255s %1023[^#]", type, first);
  
	if (stored < 2) {
		logit(log_file, "PANIC Bad configuration line for [%s]\n", s);
		bridge_mode = 1;
		xfree(type);
		xfree(first);
		return;
	}

	/* URl to redirect Squid on virus found */  
	if(strcmp(type, "redirect") == 0) {
		redirect_url = (char *) malloc (sizeof (char) * LOW_BUFF);
		if(redirect_url == NULL) {
			logit(log_file, "PANIC unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		xstrncpy(redirect_url, first, LOW_BUFF);
		xfree(type);
		xfree(first);
		return;
	}

	/* Path to chained other Squid redirector, mostly SquidGuard */
	if(strcmp(type, "squidguard") == 0) {
		squidguard = (char *) malloc (sizeof (char) * LOW_BUFF);
		if(squidguard == NULL) {
			logit(log_file, "PANIC unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		if (isPathExists(first) == 0) {
			xstrncpy(squidguard, first, LOW_BUFF);
		} else {
			logit(log_file, "Wrong path to SquidGuard, disabling.\n");
			squidguard = NULL;
		}
		xfree(type);
		xfree(first);
		return;
	}
  
	if(strcmp(type, "debug") == 0) {
		debug = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "timeout") == 0) {
		timeout = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "force") == 0) {
		force = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "stat") == 0) {
		statit = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "maxredir") == 0) {
		maxredir = atoi(first);
		xfree(type);
		xfree(first);
		return;
	}

	/* Path to log file */
	if(strcmp(type, "logfile") == 0) {
		log_file = (char *) malloc (sizeof (char) * LOW_BUFF);
		if (log_file == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		xstrncpy(log_file, first, LOW_BUFF);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "proxy") == 0) {
		proxy = (char *) malloc (sizeof (char) * MAX_CHAR);
		if(proxy == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		xstrncpy(proxy, first, MAX_CHAR);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "clamd_ip") == 0) {
		clamd_ip = (char *) malloc (sizeof (char) * LOW_CHAR);
		if (clamd_ip == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		xstrncpy(clamd_ip, first, LOW_CHAR);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "clamd_port") == 0) {
		clamd_port = (char *) malloc (sizeof (char) * LOW_CHAR);
		if(clamd_port == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		xstrncpy(clamd_port, first, LOW_CHAR);
		xfree(type);
		xfree(first);
		return;
	}

	if(strcmp(type, "clamd_local") == 0) {
		clamd_local = (char *) malloc (sizeof (char) * LOW_BUFF);
		if(clamd_local == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		xstrncpy(clamd_local, first, LOW_BUFF);
		xfree(type);
		xfree(first);
		return;
	}

	/* User Agent */
	if(strcmp(type, "useragent") == 0) {
		useragent = (char *) malloc (sizeof (char) * LOW_BUFF);
		if (useragent == NULL) {
			fprintf(stderr, "unable to allocate memory in add_to_patterns()\n");
			bridge_mode = 1;
			return;
		}
		xstrncpy(useragent, first, LOW_BUFF);
		xfree(type);
		xfree(first);
		return;
	}
	if( (strcmp(type, "regexi") == 0) || (strcmp(type, "aborti") == 0) || (strcmp(type, "contenti") == 0) || (strcmp(type, "abortcontenti") == 0)  || (strcmp(type, "whitelist") == 0)  ) {
		regex_flags |= REG_ICASE;
	}
	/* Fill the pattern type */
	if ( (strcmp(type, "abort") == 0) || (strcmp(type, "aborti") == 0) ) {
		patterns[pattc].type = ABORT;
	} else if( (strcmp(type, "content") == 0) || (strcmp(type, "contenti") == 0) ) {
		patterns[pattc].type = CONTENT;
		check_content_type = 1;
	} else if( (strcmp(type, "abortcontent") == 0) || (strcmp(type, "abortcontenti") == 0) ) {
		patterns[pattc].type = ABORTCONTENT;
		check_content_type = 1;
	} else if( (strcmp(type, "regex") == 0) || (strcmp(type, "regexi") == 0) ) {
		patterns[pattc].type = NORMAL;
		has_regex = 1;
	} else if(strcmp(type, "whitelist") == 0) {
		patterns[pattc].type = WHITELIST;
		has_regex = 1;
	} else {
		fprintf(stderr, "Bad configuration keyword: %s\n", s);
		xfree(type);
		xfree(first);
		return;
	}

	/* Fill the pattern flag */
	patterns[pattc].flag = regex_flags;

	/* Fill pattern array */	
	xstrncpy(patterns[pattc].pattern, first, MAX_CHAR);

	pattc++;

	xfree(type);
	xfree(first);

}

/*
 * Compile the collected pattern strings into an array
 * of regex_t.
 */
regex_t *
regcomp_pattern(void)
{
	int i, r = 0;
	regex_t *rxv;

	rxv = malloc(sizeof(regex_t) * pattc);
	if (rxv == NULL) {
	    logit(log_file, "PANIC unable to allocate memory in regcomp_pattern()\n");
	} else {
		for (i = 0; i < pattc; i++) {
			if ((r = regcomp(&rxv[i], patterns[i].pattern, patterns[i].flag)) != 0) {
				logit(log_file, "ERROR Invalid regex pattern: %s\n", patterns[i].pattern);
			}
		}
	}

	return rxv;
}

