/* squidclamav.c */
   
/*
  SquidClamav - A Clamav Antivirus Redirector for Squid

  This project is a modified version of the excellent Squirm Redirector
  for Squid Maintained by Chris Foote, and copyrighted
  as follow :

  Copyright (C) 1998 Chris Foote & Wayne Piekarski

  The original Squirm version used was squirm-1.0betaB. Some other parts
  are cut and paste from the ex1.c program given in the ClamAv distribution
  and are copyrighted: Copyright (C) 2002 - 2004 Tomasz Kojm

  All other code: Copyright (C) 2005-2008 Gilles Darold

  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
    
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  
  Please see the file GPL in this directory for full copyright
  information.
*/


#include"squidclamav.h"
#include"log.h"
#include"util.h"

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<unistd.h>
#include<ctype.h>

extern int debug;                         /* from main.c  */
extern char *log_file;                    /* from main.c  */

static int get_ip(char *buff, struct IP *src_address);

/* load the stdin for the redirector into
   an IN_BUFF structure 
   
   Sets in_buff.url to "" if the fields can't
   be converted
*/
int load_in_buff(char *buff, struct IN_BUFF * in_buff)
{
  int converted;
  struct IP address;
 
  address.first = 0; 
  address.second = 0; 
  address.third = 0; 

  converted = sscanf(buff, "%8192s %255s %255s %30s\n", in_buff->url, 
		     in_buff->src_address, in_buff->ident, in_buff->method);
  
  
  if(converted != 4) {
    logit(log_file, "ERROR %d = incorrect number of scanned fields\n", converted);
    return 1;
  }
  
  if (debug != 0)
	logit(log_file, "DEBUG Request:%s %s %s %s\n", in_buff->url, in_buff->src_address, in_buff->ident, in_buff->method);

  /* check the source IP address */
  if(strcmp(in_buff->src_address, "") == 0) {
    logit(log_file, "ERROR in_buff.src_address is NULL in main()\n");
    return 1;
  }


  /* all methods must be GET */
  if((strcmp(in_buff->method, "") == 0) || (strcmp(in_buff->method, "GET") != 0)) {
    if (debug != 0)
       logit(log_file, "DEBUG in_buff.method not \"GET\" (%s)\n", in_buff->method);
    return 1;
  }

  /* handle the error condition in which 4 arguments not parsed,
     in which we just print buff to stdout */
  if(xstrnlen(in_buff->url, MAX_BUFF) <= 4) {
    logit(log_file, "ERROR strlen in_buff->url = [%d] in main()\n", 
	xstrnlen(in_buff->url, MAX_BUFF));
    return 1;
  }

  /* check that the IP source address supplied is valid */
  if(get_ip(in_buff->src_address,&address) != 0) {
    logit(log_file, "ERROR in_buff.src_address not valid\n");
    return 1;
  } 
  
  return 0;
}

int url_pattern_compare(regex_t *regexv, char *url)
{
	int i = 0;  

	/* pass througth all regex pattern */
	for (i = 0; i < pattc; i++) {
		/* return -1 if string matches ABORT pattern */
		/* return -1 if string matches WHITELIST pattern */
		/* return 1 if string matches NORMAL regex pattern */
		if (regexec(&regexv[i], url, 0, 0, 0) == 0) {
			if(patterns[i].type == ABORT) {
				if (debug != 0)
					logit(log_file, "DEBUG abort matched: %s\n", url);
				return -1;
			} else if(patterns[i].type == WHITELIST) {
				if (debug != 0)
					logit(log_file, "DEBUG whitelist matched: %s\n", url);
				return -1;
			} else if(patterns[i].type == NORMAL) {
				if (debug != 0)
					logit(log_file, "DEBUG regex matched: %s\n", url);
				return 1;
			}
		}
	}

	/* return 0 otherwise */
	return 0;
}

int content_pattern_compare(regex_t *regexv, char *contenttype)
{
	int i = 0;  

	/* pass througth all regex pattern */
	for (i = 0; i < pattc; i++) {
		/* return 1 if string matches CONTENT pattern */
		/* return 0 if string matches ABORTCONTENT pattern */
		if (regexec(&regexv[i], contenttype, 0, 0, 0) == 0) {
			if(patterns[i].type == CONTENT) {
				if (debug != 0)
					logit(log_file, "DEBUG content matched: %s\n", contenttype);
				return 1;
			} else if(patterns[i].type == ABORTCONTENT) {
				if (debug != 0)
					logit(log_file, "DEBUG content abort matched: %s\n", contenttype);
				return 0;
			}
		}
	}

	/* return 0 otherwise */
	return 0;
}

int get_ip(char *src_addr, struct IP *src_address)
{
  char *ptr;
  int address;
  int i;
  char *s = (char *) malloc (sizeof (char) * MAX_CHAR);

  xstrncpy(s, src_addr, MAX_CHAR);
  
  /* split up each number from string */
  ptr = strtok(s, ".");
  if(ptr == NULL) {
    xfree(s);
    return 1;
  }
  
  /* make sure we have numbers and dots only! */
  if(strspn(s, "0123456789.") != xstrnlen(s, MAX_CHAR)) {
    xfree(s);
    return 1;
  }
  address = atoi(ptr);
  if(address < 0 || address > 255) {
    xfree(s);
    return 1;
  }
  src_address->first = (short) address;
  
  for(i = 2; i < 4; i++) {
    ptr = strtok(NULL, ".");
    if(ptr == NULL) {
      xfree(s);
      return 1;
    }
    
    /* make sure we have numbers and dots only! */
    if(strspn(s, "0123456789.") != xstrnlen(s, MAX_CHAR)) {
      xfree(s);
      return 1;
    }
    address = atoi(ptr);
    if(address < 0 || address > 255) {
      xfree(s);
      return 1;
    }
    if(i == 2)
      src_address->second = (short) address;
    if(i == 3)
      src_address->third = (short) address;
  }
  xfree(s);
  
  return 0;
}

